/*
	Columns.cpp
*/

#include "stdafx.h"
#include "Column.h"
#include "Columns.h"
#include "Glob.h"
#include "Messages.h"


HRESULT CLiteColumns::litex_error( HRESULT hr ) const
{
	CComBSTR bsDesc;
	if ( FAILED(hr) && get_description( hr, bsDesc ) )
	{
		return Error( bsDesc, IID_ILiteStatement, hr );
	}
	else
	{
		return hr;
	}
}


STDMETHODIMP CLiteColumns::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ILiteColumns
	};

	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (IsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CLiteColumns::Init( ILiteStatement* pStmt )
{
	HRESULT hRes;
	
	m_pStmt = pStmt;
	hRes = m_pStmt->get_ColumnCount( &m_nCount );
	if ( FAILED(hRes) )
	{
		return hRes;
	}
	
	SAFEARRAY* sa = SafeArrayCreateVectorEx( VT_DISPATCH, 0, m_nCount, (PVOID)&IID_IDispatch );
	if ( sa == NULL )
	{
		return E_FAIL;
	}
	
	for( LONG i=0; i<m_nCount; i++ )
	{
		CComPtr<IDispatch> pColumn;
		hRes = CLiteColumn::CreateObject( m_pStmt, i, &pColumn );
		if ( FAILED(hRes) )
		{
			break;
		}
		
		hRes = SafeArrayPutElement( sa, &i, pColumn );
		if ( FAILED(hRes) )
		{
			break;
		}
	}

	if ( FAILED(hRes) )
	{
		SafeArrayDestroy( sa );
		return hRes;
	}
	else
	{
		m_sa = sa;
	}
	
	return S_OK;
}


HRESULT CLiteColumns::CreateObject( ILiteStatement* pStmt, IDispatch** ppObj)
{
	typedef CComObject<CLiteColumns> _class;
	HRESULT hRes = E_POINTER;
	if (ppObj != NULL)
	{
		*ppObj = NULL;
		_class* p;
		hRes = _class::CreateInstance(&p);
		if (SUCCEEDED(hRes))
		{
			// If this object has ownership of the data then we need to keep it around
			hRes = p->Init(pStmt);
			if (SUCCEEDED(hRes))
			{
				hRes = p->_InternalQueryInterface( IID_IDispatch, (void**)ppObj);
			}
			
			if (FAILED(hRes))
			{
				delete p;
			}
		}
	}
	return hRes;
}

STDMETHODIMP CLiteColumns::get__NewEnum(IUnknown** pVal)
{
	CComPtr<IUnknown> pThis;
	HRESULT hRes = _InternalQueryInterface( IID_IUnknown, (void**)&pThis );
	if ( FAILED(hRes) )
	{
		return hRes;
	}

	return CLiteColumnsEnum::CreateObject( m_sa, pThis, pVal );
}

STDMETHODIMP CLiteColumns::get_Item( VARIANT vIndex, IDispatch* *pVal)
{
	if ( pVal == NULL )
	{
		return E_POINTER;
	}
	
	HRESULT hRes = S_OK;
	CComVariant v( vIndex );
	hRes = variant_normalize( v );
	if ( FAILED(hRes) ) return litex_error( hRes );
	
	switch( V_VT(&v) & VT_TYPEMASK )
	{
		case VT_INT:
		{
			CComPtr<IDispatch> pColumn;		
			hRes = SafeArrayGetElement( m_sa, (LONG*)&V_INT(&v), &pColumn );
			if ( S_OK == hRes )
			{
				hRes = pColumn->QueryInterface( IID_IDispatch, (void**)pVal );
			}
		}
		break;
		
		case VT_BSTR:
		{
			CComBSTR bsName;
			bool bFind = false;
			for( LONG i=0; i<m_nCount; i++ )
			{
				CComBSTR bsIdx( V_BSTR(&v) );
				CComPtr<IDispatch> pVColumn;
				hRes = SafeArrayGetElement( m_sa, &i, &pVColumn );
				if ( S_OK != hRes ) break;
				
				CComPtr<ILiteColumn> pColumn;
				hRes = pVColumn.QueryInterface( &pColumn );
				if ( S_OK != hRes ) break;
				
				hRes = pColumn->get_Value( &bsName );
				if ( S_OK != hRes ) break;
				
				if ( VARCMP_EQ == compare_bstr( V_BSTR(&v), bsName ) )
				{
					hRes = pColumn->QueryInterface( IID_IDispatch, (void**)pVal );
					bFind = true;
					break;
				}
			}
			
			if ( (S_OK==hRes) && !bFind )
			{
				hRes = E_UNKNOWN_COLUMN_NAME;
			}
			
		}
		break;
		
		default:
		ATLASSERT( FALSE );
		break;
	}
	
	if ( FAILED(hRes) ) return litex_error( hRes );
	return hRes;
}

STDMETHODIMP CLiteColumns::get_Count(LONG* pVal)
{
	if ( pVal == NULL )
	{
		return E_POINTER;
	}
	*pVal = m_nCount;
	return S_OK;
}

