/*
	Connection.h
*/

#ifndef __LITECONNECTION_H_
#define __LITECONNECTION_H_

#include "resource.h"       // main symbols
#include "LiteX.h"
#include "ILiteBusy_CP.H"
#include "ILiteProgress_CP.H"

#define DEF_PROGRESS_PERIOD 0

class ATL_NO_VTABLE CLiteConnection : 
	public CComObjectRootEx<CComObjectThreadModel>,
	public CComCoClass<CLiteConnection, &CLSID_LiteConnection>,
	public ISupportErrorInfo,
	public ILiteConnectionPrivate,
	public IDispatchImpl<ILiteConnection, &IID_ILiteConnection, &LIBID_LITEXLib>,
	public IConnectionPointContainerImpl<CLiteConnection>,
	public CProxyILiteBusy<CLiteConnection>,
	public CProxyILiteProgress<CLiteConnection>,
/*
	Knowledge Base  
	How To Enable ActiveX Control Event Handling on a Web Page
	PSS ID Number: 200839
*/	
	public IProvideClassInfo2Impl<&CLSID_LiteConnection, &IID_ILiteProgress, &LIBID_LITEXLib>

{
public:

	CLiteConnection()
	{
		m_db = NULL;
#ifndef _NO_UNACC_STUFF
		m_vUnaccContext = NULL;
#endif
		m_nProgressPeriod = DEF_PROGRESS_PERIOD;
	}

	void FinalRelease()
	{
		if ( m_db != NULL )
		{
			sqlite3_interrupt( m_db );
#ifndef _NO_UNACC_STUFF
			sqlite3_unregister_unacc_functions( m_db, m_vUnaccContext );
#endif
			sqlite3_busy_handler( m_db, NULL, 0 );
			sqlite3_progress_handler( m_db, DEF_PROGRESS_PERIOD, NULL, 0 );
			
			sqlite3_close( m_db );
		}
	}

DECLARE_REGISTRY_RESOURCEID(IDR_LITECONNECTION)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CLiteConnection)
	COM_INTERFACE_ENTRY(ILiteConnection)
	COM_INTERFACE_ENTRY(ILiteConnectionPrivate)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CLiteConnection)
	CONNECTION_POINT_ENTRY( IID_ILiteProgress )
	CONNECTION_POINT_ENTRY( IID_ILiteBusy )
END_CONNECTION_POINT_MAP()


// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// ILiteConnectionPrivate
public:

	STDMETHOD(GetDB)( /*[out,retval]*/ void* *pDB );

// ILiteConnection
public:
	STDMETHOD(BatchExecute)( /*[in]*/ BSTR bsSql );
	STDMETHOD(OpenInMemory)();
	STDMETHOD(get_LastInsertRowid)( /*[out,retval]*/ VARIANT *pVal );
	STDMETHOD(get_Changes)( /*[out,retval]*/ LONG *pnVal );
	STDMETHOD(Execute)(/*[in]*/ BSTR bsCommand, /*[in]*/ LPSAFEARRAY *params, /*[out,retval]*/ VARIANT *pVal );
	STDMETHOD(Prepare)(/*[in]*/ BSTR bsCommand, /*[out,retval]*/ IDispatch* *ppStmt);
	STDMETHOD(Close)();
	STDMETHOD(Open)(/*[in,optional,defaultvalue("")]*/ BSTR bsPath);
	STDMETHOD(get_Path)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Path)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_Version)( /*in,defaultvalue(VARIANT_TRUE)*/ VARIANT_BOOL bLong, /*[out, retval]*/ VARIANT *pVal);
	STDMETHOD(get_ProgressPeriod)(/*[out, retval]*/ LONG *pnPeriod );
	STDMETHOD(put_ProgressPeriod)(/*[in]*/ LONG nPeriod );
	STDMETHOD(Interrupt)();

protected:

	CComBSTR m_bsPath;
	sqlite3* m_db;
#ifndef _NO_UNACC_STUFF
	void* m_vUnaccContext;
#endif
	LONG m_nProgressPeriod;

	HRESULT sqlite_error( int ) const;
	HRESULT litex_error( HRESULT ) const;
	HRESULT sqlite_error( int, const char* ) const;

	HRESULT create_statement( CComPtr<ILiteStatement>& );
	
protected:

	static int busy_callback( void*, int );
	static int progress_callback( void * );

};

#endif //__LITECONNECTION_H_
