/*
	Rows.cpp
*/

#include "stdafx.h"
#include "LiteX.h"
#include "RowsEnum.h"
#include "Rows.h"
#include "Glob.h"

static const LONG DEF_SIZE = 100;
static const LONG DEF_GROW = DEF_SIZE / 2;

// CCLiteRows

STDMETHODIMP CLiteRows::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ILiteRows
	};

	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (IsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CLiteRows::build_array_of_rows( LONG nMaxRec )
{
	ULONG nSize = DEF_SIZE;
	SAFEARRAY* sa = SafeArrayCreateVectorEx( VT_DISPATCH, 0, nSize, (PVOID)&IID_IDispatch );
	if ( sa == NULL )
	{
		return E_OUTOFMEMORY;
	}
	
	VARIANT_BOOL bDone;
	CComVariant row;
	LONG nElements = 0;
	HRESULT hRes = S_OK;
	
	if ( nMaxRec <= 0 )
	{
		nMaxRec = LONG_MAX; // there's allways upper limit
	}
	
	hRes = m_pStmt->Reset();
	if ( S_OK == hRes )
	{
		while( true )
		{
			hRes = m_pStmt->Step( 1, &bDone );
			if ( FAILED(hRes) ) break;
			if ( bDone == VARIANT_TRUE ) break;
			
			if ( nElements >= nMaxRec )
			{
				break;
			}
			
			if ( (ULONG)nElements >= nSize )
			{
				SAFEARRAYBOUND sab;
				sab.cElements = nSize + DEF_GROW;
				sab.lLbound = 0;
				
				hRes = SafeArrayRedim( sa, &sab );
				if ( FAILED(hRes) ) break;
				
				nSize = sab.cElements;
			}
			
			hRes = m_pStmt->get_Row( lxCollection, &row );
			if ( FAILED(hRes) ) break;
			
			hRes = SafeArrayPutElement( sa, &nElements, V_DISPATCH(&row) );
			if ( FAILED(hRes) ) break;
			
			nElements++;
		}
	}
	
	if ( FAILED(hRes) )
	{
		SafeArrayDestroy( sa );
	}
	else
	{
		SAFEARRAYBOUND sab;
		sab.cElements = nElements;
		sab.lLbound = 0;
		hRes = SafeArrayRedim( sa, &sab );
		
		if ( S_OK == hRes )
		{
			m_sa = sa;
		}
	}
	
	return hRes;
}

HRESULT CLiteRows::Init( bool bStatic, LONG nMaxRecords, ILiteStatement* pStmt )
{
	HRESULT hRes = S_OK;
	m_bStatic = bStatic;
	m_pStmt = pStmt;
	if ( bStatic )
	{
		hRes = build_array_of_rows( nMaxRecords );
	}
	
	return hRes;
}

HRESULT CLiteRows::Init( SAFEARRAY* sa )
{
	m_bStatic = true;
	HRESULT hRes = SafeArrayCopy( sa, &m_sa );	
	return hRes;
}

STDMETHODIMP CLiteRows::get__NewEnum( IUnknown **pUnkEnum)
{
	if ( pUnkEnum == NULL )
	{
		return E_POINTER;
	}
	
	HRESULT hRes;
	if ( m_bStatic )
	{
		CComPtr<IUnknown> pThis;
		hRes = _InternalQueryInterface( IID_IUnknown, (void**)&pThis );
		if ( S_OK == hRes )
		{
			hRes = CLiteRowsEnum::CreateObject( m_sa, pThis, pUnkEnum );
		}
	}
	else
	{
		hRes = CLiteRowsDynEnum::CreateObject( m_pStmt, pUnkEnum );
	}
	return hRes;
}

HRESULT CLiteRows::CreateObject( bool bStatic, LONG nMaxRecords, ILiteStatement* pStmt, IDispatch** ppObj)
{
	typedef CComObject<CLiteRows> _class;
	HRESULT hRes = E_POINTER;
	if (ppObj != NULL)
	{
		*ppObj = NULL;
		_class* p;
		hRes = _class::CreateInstance(&p);
		if (SUCCEEDED(hRes))
		{
			// If this object has ownership of the data then we need to keep it around
			hRes = p->Init(bStatic,nMaxRecords,pStmt);
			if (SUCCEEDED(hRes))
			{
				hRes = p->_InternalQueryInterface( IID_IDispatch, (void**)ppObj);
			}
			
			if (FAILED(hRes))
			{
				delete p;
			}
		}
	}
	return hRes;
}

HRESULT CLiteRows::CreateObject( SAFEARRAY* sa, IDispatch** ppObj)
{
	typedef CComObject<CLiteRows> _class;
	HRESULT hRes = E_POINTER;
	if (ppObj != NULL)
	{
		*ppObj = NULL;
		_class* p;
		hRes = _class::CreateInstance(&p);
		if (SUCCEEDED(hRes))
		{
			// If this object has ownership of the data then we need to keep it around
			hRes = p->Init(sa);
			if (SUCCEEDED(hRes))
			{
				hRes = p->_InternalQueryInterface( IID_IDispatch, (void**)ppObj);
			}
			
			if (FAILED(hRes))
			{
				delete p;
			}
		}
	}
	return hRes;
}



STDMETHODIMP CLiteRows::get_Count( VARIANT* pVal)
{
	HRESULT hRes = S_OK;
	if ( m_bStatic )
	{
		LONG nCount = 0;
		hRes = SafeArrayGetUBound( m_sa, 1, &nCount );
		if ( S_OK == hRes )
		{
			nCount++;
			CComVariant val;
			val = nCount;
			hRes = val.Detach( pVal );
		}
	}
	else
	{
		hRes = m_pStmt->get_RowCount( pVal );
	}
	return hRes;	
}

STDMETHODIMP CLiteRows::get_Item(LONG nIndex, IDispatch** pVal)
{
	HRESULT hr;
	if ( m_bStatic )
	{
		hr = SafeArrayGetElement( m_sa, &nIndex, pVal );
	}
	else
	{
		ATLASSERT(nIndex==-1);
		CComVariant val;
		hr = m_pStmt->get_Row( lxCollection, &val );
		if ( S_OK == hr )
		{
			ATLASSERT( V_VT(&val) == VT_DISPATCH );
			hr = V_DISPATCH(&val)->QueryInterface( IID_IDispatch, (void**)pVal );
		}
	}

	return hr;
}
