/*
	RowsEnum.cpp
*/

#include "stdafx.h"
#include "LiteX.h"
#include "Statement.h"
#include "RowsEnum.h"

STDMETHODIMP CLiteRowsDynEnumImpl::Next(ULONG celt, VARIANT* rgelt,
	ULONG* pceltFetched)
{
	if (pceltFetched != NULL)
	{
		*pceltFetched = 0;
	}
	
	if (celt == 0)
		return E_INVALIDARG;
	if (rgelt == NULL || (celt != 1 && pceltFetched == NULL))
		return E_POINTER;
	
	VARIANT* pVar = rgelt;
	CComVariant var;
	ULONG i, nCounter = 0;
	VARIANT_BOOL bDone;
	HRESULT hr;
	
	for( i=0; i<celt; pVar++,i++ )
	{
		bDone = VARIANT_FALSE;
		hr = m_pStmt->Step( 1, &bDone );
		if ( FAILED(hr) ) break;
		
		if ( bDone == VARIANT_TRUE ) break;

		hr = m_pStmt->get_Row( lxCollection, &var );
		if ( FAILED(hr) ) break;
		
		hr = var.ChangeType( VT_DISPATCH );
		if ( FAILED(hr) ) break;
		
		hr = var.Detach( pVar );
		if ( FAILED(hr) ) break;
		
		nCounter++;
	}
	
	if ( FAILED(hr) )
	{
		return hr;
	}
	else
	{
		if ( pceltFetched != NULL )
		{
			*pceltFetched = nCounter;
		}
		
		return (celt==nCounter)? S_OK : S_FALSE;
	}
}

STDMETHODIMP CLiteRowsDynEnumImpl::Skip(ULONG celt)
{
	if (celt == 0)
	{
		return E_INVALIDARG;
	}

	VARIANT_BOOL bDone = VARIANT_FALSE;
	HRESULT hRes = m_pStmt->Step(celt,&bDone);
	if ( FAILED(hRes) )
	{
		return hRes;
	}
	else
	{
		return (bDone == VARIANT_FALSE)? S_OK : S_FALSE;
	}
}

STDMETHODIMP CLiteRowsDynEnumImpl::Reset()
{
	return m_pStmt->Reset();
}

STDMETHODIMP CLiteRowsDynEnumImpl::Clone(IEnumVARIANT** ppEnum)
{
	CComPtr<IUnknown> pUnkEnum;
	HRESULT hRes = CLiteRowsDynEnum::CreateObject( m_pStmt.p, &pUnkEnum.p );
	if ( FAILED(hRes) )
	{
		return hRes;
	}
	else
	{
		hRes = pUnkEnum->QueryInterface( IID_IEnumVARIANT, (void**)ppEnum );
		return hRes;
	}
	
}

HRESULT CLiteRowsDynEnumImpl::Init( ILiteStatement* pStmt )
{
	m_pStmt = pStmt;
	return S_OK;
}

//

STDMETHODIMP CLiteRowsDynEnum::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IEnumVARIANT
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (IsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


HRESULT CLiteRowsDynEnum::CreateObject( ILiteStatement* pStmt, IUnknown** ppEnum)
{
	typedef CComObject<CLiteRowsDynEnum> _class;
	HRESULT hRes = E_POINTER;
	if (ppEnum != NULL)
	{
		*ppEnum = NULL;
		_class* p;
		hRes = _class::CreateInstance(&p);
		if (SUCCEEDED(hRes))
		{
			// If this object has ownership of the data then we need to keep it around
			hRes = p->Init(pStmt);
			if (SUCCEEDED(hRes))
			{
				hRes = p->_InternalQueryInterface( IID_IUnknown, (void**)ppEnum);
			}
			
			if (FAILED(hRes))
			{
				delete p;
			}
		}
	}
	return hRes;
}
