/*
	Statement.h
*/

#ifndef __LITESTATEMENT_H_
#define __LITESTATEMENT_H_

#include "resource.h"       // main symbols

class ATL_NO_VTABLE CLiteStatement : 
	public CComObjectRootEx<CComObjectThreadModel>,
	public CComCoClass<CLiteStatement, &CLSID_LiteStatement>,
	public ISupportErrorInfo,
	public IDispatchImpl<ILiteStatement, &IID_ILiteStatement, &LIBID_LITEXLib>,
	public IProvideClassInfo2Impl<&CLSID_LiteStatement, NULL, &LIBID_LITEXLib>
{
public:

	CLiteStatement()
	{
		m_eState = lxCreated;
		m_stmt = NULL;
	}

	void FinalRelease()
	{
		if ( m_eState == lxPrepared )
		{
			int nRes;
			nRes = sqlite3_finalize( m_stmt );
		}
	}

DECLARE_REGISTRY_RESOURCEID(IDR_LITESTATEMENT)
DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CLiteStatement)
	COM_INTERFACE_ENTRY(ILiteStatement)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// ILiteStatement
public:

	STDMETHOD(Execute)();
	STDMETHOD(BindParameters)(/*[in]*/ LPSAFEARRAY *params );
	STDMETHOD(get_RowCount)( /*[out,retval]*/ VARIANT *pVal );
	STDMETHOD(get_Row)( /*[in,defaultvalue(lxDefault)]*/ LxRowEnum eMode, /*[out,retval]*/ VARIANT *pVal );
	STDMETHOD(get_ParameterName)( /*[in]*/ LONG nIndex, /*[out,retval]*/ BSTR *bsName );
	STDMETHOD(get_ParameterCount)(/*[out, retval]*/ LONG *pVal);
	STDMETHOD(get_Done)( /*[out,retval]*/ VARIANT_BOOL *pbDone );
	STDMETHOD(get_State)(/*[out, retval]*/ LxStateEnum *pVal);
	STDMETHOD(Close)();
	STDMETHOD(Step)( /*[in,defaultvalue(1)*/LONG nStep, /*[out,retval]*/ VARIANT_BOOL *pbDone );
	STDMETHOD(Reset)();
	STDMETHOD(get_ColumnValue)(/*[in]*/ VARIANT idx, /*[in, optional]*/ LxTypeEnum eType, /*[out, retval]*/ VARIANT *pVal );
	STDMETHOD(get_ColumnName)(/*[in]*/ LONG nColumn, /*[out, retval]*/ BSTR *pVal);
	STDMETHOD(get_ColumnType)( /*[in]*/ VARIANT idx, /*[out,retval]*/ LxTypeEnum* eType );
	STDMETHOD(get_ColumnCount)(/*[out, retval]*/ LONG *pVal);
	STDMETHOD(Prepare)( /*in,optional,defaultvalue("")*/ BSTR bsCommand );
	STDMETHOD(BindParameter)(/*[in]*/ VARIANT idx, /*[in,optional]*/ VARIANT vValue, /*[in,optional,defaultvalue(lxUnknown)]*/ LxTypeEnum eType );
	STDMETHOD(get_CommandText)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_CommandText)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_ActiveConnection)(/*[out, retval]*/ IDispatch* *pVal);
	STDMETHOD(put_ActiveConnection)(/*[in]*/ IDispatch *pVal);
	STDMETHOD(get_Rows)( /*[in,defaultvalue(VARIANT_FALSE)] */ VARIANT_BOOL bStatic, /*[in,defaultvalue(0)]*/ LONG nMaxRecords , /*[out,retval]*/ IDispatch** pVal );
	STDMETHOD(get_Columns)( /*[out,retval]*/ IDispatch** pVal );
	STDMETHOD(get_Parameters)( /*[out,retval]*/ IDispatch** pVal);

protected:

	LxStateEnum m_eState;
	CComBSTR m_bsCommand;
	CComPtr<ILiteConnectionPrivate> m_pConnection;
	sqlite3_stmt* m_stmt;
	bool m_bDone;

	HRESULT bind_blob( CComPtr<IStream>&, LONG, int& );
	HRESULT bind_blob( SAFEARRAY*, LONG, int&  );

	HRESULT sqlite_error( sqlite3* ) const;
	HRESULT sqlite_error( int ) const;
	HRESULT litex_error( HRESULT ) const;

	LxTypeEnum get_column_type( int ) const;
	HRESULT get_column_index( const VARIANT&, int& ) const;
	HRESULT get_column_name( LONG, CComBSTR& ) const;

	HRESULT get_parameter_index( const VARIANT&, int& ) const;
	HRESULT create_row( LONG, CComVariant& v );

};

#endif //__LITESTATEMENT_H_
