/*
	This is sample application that demonstrates how to use LiteX++ library.
	It tries to use nearly all classes and methods from this library.
	Please try compile this application with Unicode or without Unicode support.
	
	To use LiteX++ library you must include LiteX.hpp and LiteX.cpp files in your project.
*/

#include "stdafx.h"
#include <string>
#include <iostream>
#include <sstream>
#include "tstd.h"
#include "LiteX.hpp"
//#include <cmath>
#include <cstdlib>
#include <ctime>

// VC++ 6
#if _MSC_VER <= 1200
namespace std
{
	// ostream int64 overloads
	template< class int_type, class stream_type >
	int _format_int( const stream_type& stream, int_type n, stream_type::char_type*& psz )
	{
		stream_type::fmtflags flags = stream.flags();
		if( flags & stream_type::hex ) {
			*psz++ = (stream_type::char_type) '0';
			*psz++ = (stream_type::char_type) 'x';
			return 16;
		}
		else if( flags & stream_type::oct ) {
			*psz++ = (stream_type::char_type) '0';
			return 8;
		}
		else if( flags & stream_type::showpos && n > 0 )
			*psz++ = (stream_type::char_type) '+';
		return 10;
	}

	ostream& operator <<( ostream& os, __int64 n )
	{
		char szBuf[ 32 ];
		char* psz = szBuf;
		int nBase = _format_int( os, n, psz );
		_i64toa( n, psz, nBase );
		return os << szBuf;
	}

	wostream& operator <<( wostream& os, __int64 n )
	{
		wchar_t szBuf[ 32 ];
		wchar_t* psz = szBuf;
		int nBase = _format_int( os, n, psz );
		_i64tow( n, psz, nBase );
		return os << szBuf;
	}

	ostream& operator <<( ostream& os, unsigned __int64 n )
	{
		char szBuf[ 32 ];
		char* psz = szBuf;
		int nBase = _format_int( os, n, psz );
		_ui64toa( n, psz, nBase );
		return os << szBuf;
	}

	wostream& operator <<( wostream& os, unsigned __int64 n )
	{
		wchar_t szBuf[ 32 ];
		wchar_t* psz = szBuf;
		int nBase = _format_int( os, n, psz );
		_ui64tow( n, psz, nBase );
		return os << szBuf;
	}
};
#endif

using namespace litex;
using namespace std;

static const int BLOCK_SIZE = 0x4321;
static const int TABLE_SIZE = 50000;

static void prepare_blob( BYTE* pBlob, int nBlobSize )
{
	for( int i=0; i<nBlobSize; i++ )
	{
		pBlob[i] = static_cast<BYTE>(rand() % 256);
	}
}

static void show_column( tostream& sout, const SQLiteStatement& stmt )
{
	sout << 
	_T("count=") << stmt.get_DataCount() << _T(" -->" ) <<
	_T(" a=") << stmt.get_ColumnInt(0) <<
	_T(" b=") << stmt.get_ColumnText(1) <<
	_T(" c=") << stmt.get_ColumnInt(2);
	
	if ( stmt.IsNull(3) )
	{
		sout << _T(" d=NULL");	
	}
	else
	{
		sout << _T(" d=") << stmt.get_ColumnDouble(3);
		
	}
	sout << _T(" e=") << stmt.get_ColumnInt64(4);

	int nBlockSize;
	const BYTE* pRetBlob = reinterpret_cast<const BYTE*>(stmt.get_ColumnBlob(5,nBlockSize));
	sout << _T(" blob size=") << nBlockSize << endl;
}

static void worker( tostream& sout )
{
	sout << _T("Hello from LiteX++ sample console application!") << endl <<
	_T("LiteX++ is a simple C++ wrapper around SQLite3 native API.") << endl << endl;
	
	sout << _T("SQLite3 library version string: ") << SQLiteConnection::get_VersionString() << endl;
	sout << _T("SQLite3 library version number: ") << SQLiteConnection::get_VersionNumber() << endl << endl;
	
	BYTE* pBlock = new BYTE[BLOCK_SIZE];
	int nBlockSize;
	
	try
	{
		int i;
		sout << _T("Creating in-memory database. Preparing database structure.") << endl;
		SQLiteConnection db( SQLiteConnection::MEMORY_DB );
		if ( db )
		{
			sout << _T("Database is open.") << endl;
		}
		db.ExecuteNonQuery( _T("CREATE TABLE Test( a INTEGER PRIMARY KEY, b TEXT, c INTEGER, d DOUBLE, e LONG INTEGER, f BLOB )") );
		db.BatchExecute( _T("CREATE INDEX d ON Test( d DESC ); CREATE INDEX e ON Test( e )") );
		
		sout << _T("Preparing Test table.") << endl;
		db.BeginTransaction();
		SQLiteStatement stmt = db.Prepare( _T("INSERT INTO Test(b,c,d,e,f) VALUES(?,?,?,:long_param,:blob_param)") );
		sout << _T("Number of parameters: ") << stmt.get_ParameterCount() << endl;
		sout << _T("Name of fourth parameter: ") << stmt.get_ParameterName(4) << endl;
		
		for( i=1; i<=TABLE_SIZE; i++ )
		{
			tostringstream ss;
			ss << _T("-=<") << hex << rand() << _T(">=-");
			
			stmt.BindText( 1, ss.str() );
			stmt.BindInt( 2, rand() );
			
			if ( i%2 )
			{
				stmt.BindDouble( 3, rand()/2.7896 );
			}
			else
			{
				stmt.BindNull( 3 );
			}
			
			stmt.BindInt64( _T(":long_param"), MAKELONG( rand() , rand() ) );
			
			nBlockSize = rand() % BLOCK_SIZE;
			prepare_blob( pBlock, nBlockSize );
			stmt.BindBlob( _T(":blob_param"), pBlock, nBlockSize );
			
			stmt.Execute();
			sout << _T('.');
		}
		
		stmt.Finalize();
		db.CommitTransaction();
		sout << endl << _T("Preparing complete.") << endl <<
		_T("Last insert ID: ") << db.get_LastInsertRowid() << endl <<
		_T("Total changes: ") << db.get_TotalChanges() << endl << endl;
		
		sout << _T("Dumping table data.") << endl;
		if ( !stmt.operator bool() )
		{
			stmt.Prepare( _T("SELECT a as 'Hello from a',b as 'Hello from b',c as 'Hell from c',d as 'Hello from d',e,f FROM Test ORDER BY e DESC") );
		}
		
		sout << 
		_T("Number of columns: ") << stmt.get_ColumnCount() << endl <<
		_T("Column 0: ") << stmt.get_ColumnName(0) << _T(" ,") << stmt.get_ColumnDecltype(0) << endl <<
		_T("Column 1: ") << stmt.get_ColumnName(1) << _T(" ,") << stmt.get_ColumnDecltype(1) << endl <<
		_T("Column 2: ") << stmt.get_ColumnName(2) << _T(" ,") << stmt.get_ColumnDecltype(2) << endl <<
		_T("Column 3: ") << stmt.get_ColumnName(3) << _T(" ,") << stmt.get_ColumnDecltype(3) << endl <<
		_T("Column 4: ") << stmt.get_ColumnName(4) << _T(" ,") << stmt.get_ColumnDecltype(4) << endl <<
		_T("Column 5: ") << stmt.get_ColumnName(5) << _T(" ,") << stmt.get_ColumnDecltype(5) << endl << endl;
		
		while( stmt.Step() )
		{
			show_column( sout, stmt );
		}
		stmt.Finalize();
		sout << endl << _T("Dump complete.") << endl << _T("Random seeking on table.") << endl;
		
		for( i=1; i<100; i++ )
		{
			int idx = 1 + (rand() % TABLE_SIZE);
			sout << _T("Index: ") << idx << endl;
			
			tostringstream ss;
			
			ss << _T("SELECT b FROM TEST WHERE a=") << idx;
			sout << _T("b: ") << db.ExecuteScalarText( ss.str() ) << endl;
			
			ss.seekp(0, ios_base::beg);
			ss << _T("SELECT d FROM TEST WHERE a=") << idx;
			sout << _T("d: ") << db.ExecuteScalarDouble( ss.str() ) << endl;
			
			ss.seekp(0, ios_base::beg);
			ss << _T("SELECT e FROM TEST WHERE a=") << idx;
			sout << _T("e: ") << db.ExecuteScalarInt64( ss.str() ) << endl << endl;
		}
		
		sout << _T("Test complete.") << endl;
		db.Close();
		
		if ( !db.operator bool() )
		{
			sout << _T("Database closed.") << endl;
		}
	}
	catch( SQLiteException& ex1 )
	{
		tcerr << _T("SQLite exception.") << endl <<
		_T("Error code: ") << ex1.get_ErrorCode() << endl <<
		_T("Error message: ") << ex1.get_Message() << endl;
	}
	catch( SQLiteRuntimeException& ex2 )
	{
		tcerr << _T("SQLite runtime exception.") << endl <<
		_T("Error message: ") << ex2.get_Message() << endl;
	}
	catch( std::exception& )
	{
		tcerr << _T("STL exception") << endl;
	}
	
	delete pBlock;
	sout << _T("Bye.") << endl;	
}

int _tmain(int argc, _TCHAR* argv[])
{
#ifdef _LITEX_WITH_DELAYLOAD
	HRESULT hRes = delayload::load_library();
	if ( FAILED(hRes) )
	{
		return hRes;
	}
#endif

	srand( (unsigned)time( NULL ) );
	
	if ( argc > 1 )
	{ // do not show any progress messages, benchmark mode
		tonullstream snull;
		worker( snull );
	}
	else
	{
		worker( tcout );
	}
	
#ifdef _LITEX_WITH_DELAYLOAD
	delayload::free_library();
#endif	
	return 0;
}

