/*
	tstd.h
*/

#pragma once

namespace std
{
	#ifdef _UNICODE

	#define tcout wcout
	#define tcerr wcerr
	#define tostream wostream
	#define tistream wistream
	#define tonullstream wonullstream
	#define todbgstream wodbgstream

	#else

	#define tcout cout
	#define tcerr cerr
	#define tostream ostream
	#define tistream istream
	#define tonullstream onullstream
	#define todbgstream odbgstream

	#endif

	//typedef basic_ostream< TCHAR > tostream;
	//typedef basic_istream< TCHAR > tistream;
	typedef basic_string< TCHAR > tstring;
	typedef basic_istringstream< TCHAR > tistringstream;
	typedef basic_ostringstream< TCHAR > tostringstream;
	typedef basic_stringbuf< TCHAR > tstringbuf;


	// null stream buffer class
	template <class charT, class traits = std::char_traits<charT> >
	class basic_nullbuf : public basic_streambuf<charT, traits>
	{
		typedef basic_streambuf<charT, traits>  base_type;

		public:

		typedef typename base_type::char_type    char_type;
		typedef typename base_type::traits_type  traits_type;
		typedef typename base_type::int_type     int_type;
		typedef typename base_type::pos_type     pos_type;
		typedef typename base_type::off_type     off_type;

		protected:

		virtual streamsize xsputn( char_type const* /*s*/, streamsize n )
		{
			return n;
		} // "s" is unused

		virtual int_type overflow(int_type c)
		{
			// just ignore the character
			return traits::not_eof(c);
		}
	};

	typedef basic_nullbuf<char>      nullbuf;
	typedef basic_nullbuf<wchar_t>  wnullbuf;

	// generic null output stream class
	template <class charT, class traits = char_traits<charT> >
	class basic_onullstream
		: private basic_nullbuf<charT, traits>,
		  public basic_ostream<charT, traits>
	{
		typedef basic_nullbuf<charT, traits> base_buff;
		typedef basic_ostream<charT, traits> base_type;

		public:
		basic_onullstream() :base_type(this)
		{}
	};

	typedef basic_onullstream<char>      onullstream;
	typedef basic_onullstream<wchar_t>  wonullstream;

	// ANSI/UNICODE overloaded OutputDebugString()
	inline void _OutDbgString( const char* pszText)
	{
		::OutputDebugStringA( pszText );
	}

	inline void _OutDbgString( const wchar_t* pszText)
	{
		::OutputDebugStringW( pszText );
	}
	
	///////////////////////////////////
	// basic_streambuf class, that writes output to the debugger
	template <class E, class T = char_traits<E>, int BUF_SIZE = 512 >
	class basic_dbgstreambuf : public basic_streambuf< E, T >
	{
		typedef basic_streambuf< E, T >  base_type;

		public:

		typedef typename base_type::char_type    char_type;
		typedef typename base_type::traits_type  traits_type;
		typedef typename base_type::int_type     int_type;
		typedef typename base_type::pos_type     pos_type;
		typedef typename base_type::off_type     off_type;

		basic_dbgstreambuf() :base_type()
		{
			psz = new char_type[ BUF_SIZE ];
			setbuf( psz, BUF_SIZE );
			// leave place for single char + 0 terminator
			setp( psz, psz + BUF_SIZE - 2 );
		}

		virtual ~basic_dbgstreambuf()
		{
			delete psz;
		}

		protected:

		virtual int_type overflow( int_type c = T::eof() )
		{
			char_type* plast = pptr();
			if( c != T::eof() )
				// add c to buffer
				*plast++ = c;
			*plast = char_type();

			_OutDbgString( pbase() );
			setp( pbase(), epptr() );

			return c != T::eof() ? T::not_eof( c ) : T::eof();
		}

		virtual int sync()
		{
			overflow();
			return 0;
		}

		virtual streamsize xsputn(const char_type* pch, streamsize n)
		{
			streamsize nMax, nPut;
			for( nPut = 0; 0 < n; )
				if( pptr() != 0 && 0 < (nMax = epptr() - pptr()) ) {
					if( n < nMax )
						nMax = n;
					traits_type::copy( pptr(), pch, nMax );
					
					// Sync if string contains LF
					bool bSync = traits_type::find( pch, nMax, traits_type::to_char_type( '\n' ) ) != NULL;
					pch += nMax, nPut += nMax, n -= nMax, pbump(nMax);
					if( bSync )
						sync();
				}
				else if (traits_type::eq_int_type(traits_type::eof(),
					overflow(traits_type::to_int_type(*pch))))
					break;
				else
					++pch, ++nPut, --n;
			return (nPut);
		}

		protected:

		char_type* psz;
	};

	typedef basic_dbgstreambuf<char> dbgstreambuf;
	typedef basic_dbgstreambuf<wchar_t> wdbgstreambuf;

	// generic debug output stream class
	template <class charT, class traits = char_traits<charT> >
	class basic_odbgstream
		: private basic_dbgstreambuf<charT, traits>,
		  public basic_ostream<charT, traits>
	{
		typedef basic_nullbuf<charT, traits> base_buff;
		typedef basic_ostream<charT, traits> base_type;

		public:
		basic_odbgstream() :base_type(this)
		{}
	};

	typedef basic_odbgstream<char> odbgstream;
	typedef basic_odbgstream<wchar_t> wodgstream;

};
