#comments-start
	LiteEvents
	How to handle LiteX Automation events in AutoIt3
#comments-end

#include <Misc.au3>
#include <Array.au3>
#include <File.au3>

Global Const $DB_FILE = _TempFile()
Global Const $MAX_REC = 44444
Global $g_oComError

Func status_text($s)
	ConsoleWrite($s & @CRLF)
EndFunc   ;==>status_text

Func com_err_func()
	Dim $sMsg = "COM Error: " & Hex($g_oComError.number, 8)
	status_text($sMsg)
	$sMsg = "Source: " & $g_oComError.source
	status_text($sMsg)
	$sMsg = "Error description #1: " & $g_oComError.windescription
	status_text($sMsg)
	$sMsg = "Error  description #2: " & $g_oComError.description
	status_text($sMsg)
	If Not @Compiled Then
		$sMsg = "Line: " & $g_oComError.scriptline
		status_text($sMsg)
	EndIf

	Return SetError(1, $g_oComError.number, False)
EndFunc   ;==>com_err_func

Func ILiteProgress1_Progress($bAbort)
	status_text("Progress event #1.")
EndFunc   ;==>ILiteProgress1_Progress

Func ILiteBusy1_Busy($nTimes, $bAbort)
	status_text("Busy event #1: " & $nTimes & ".")
EndFunc   ;==>ILiteBusy1_Busy

Func ILiteProgress2_Progress($bAbort)
	status_text("Progress event #2.")
EndFunc   ;==>ILiteProgress2_Progress

Func ILiteBusy2_Busy($nTimes, $bAbort)
	status_text("Busy event #2: " & $nTimes & ".")
EndFunc   ;==>ILiteBusy2_Busy


$g_oComError = ObjEvent("AutoIt.Error", "com_err_func")

$oConnection1 = ObjCreate("LiteX.LiteConnection")
If @error Then
	status_text("Fail to create LiteX Connection object #1.")
	Exit 1
EndIf

$oConnection2 = ObjCreate("LiteX.LiteConnection")
If @error Then
	status_text("Fail to create LiteX Connection object #2.")
	Exit 2
EndIf


status_text("Hello from SQlite3 version " & $oConnection1.Version & " .")
status_text("Hello from SQlite3 version " & $oConnection2.Version (False) & " again.")

$g_oLiteBusy1 = ObjEvent($oConnection1, "ILiteBusy1_", "ILiteBusy")
$g_oLiteProgress1 = ObjEvent($oConnection1, "ILiteProgress1_", "ILiteProgress")

$g_oLiteBusy2 = ObjEvent($oConnection2, "ILiteBusy2_", "ILiteBusy")
$g_oLiteProgress2 = ObjEvent($oConnection2, "ILiteProgress2_", "ILiteProgress")

status_text("Opening database")
$oConnection1.Open ($DB_FILE)
$oConnection2.Open ($DB_FILE)

status_text("Creating database structure.")
$oConnection1.BatchExecute ("PRAGMA page_size=512;PRAGMA CODEPAGE='UTF-8';CREATE TABLE Test( a INTEGER PRIMARY KEY,b);CREATE INDEX idxb ON Test(b)")
$oConnection2.BatchExecute ("PRAGMA page_size=512")

status_text("Simulating locking.")
$oConnection2.Execute("BEGIN TRANSACTION")
$oConnection1.Execute("BEGIN TRANSACTION")
$oConnection2.Execute("INSERT INTO Test(b) VALUES (111)")
$oConnection1.Execute("INSERT INTO Test(b) VALUES (222)")
$oConnection1.Execute("ROLLBACK TRANSACTION")
$oConnection2.Execute("ROLLBACK TRANSACTION")

status_text("Inserting data phase 1.")
$oConnection1.Execute("BEGIN TRANSACTION")
$oStmt = $oConnection1.Prepare ("INSERT INTO Test(b) VALUES (?)")
For $i = 1 To $MAX_REC
	;status_text( "Iteration " & $i )
	$oStmt.BindParameters (Random(0, $MAX_REC))
	$oStmt.Execute()
Next
$oStmt.Close ()
$oConnection1.Execute("COMMIT TRANSACTION")

status_text("Inserting data phase 2.")
$oConnection2.Execute("BEGIN TRANSACTION")
$oStmt = $oConnection2.Prepare ("INSERT INTO Test(b) VALUES (?)")
For $i = 1 To $MAX_REC
	;status_text( "Iteration " & $i )
	$oStmt.BindParameters (Random($MAX_REC, 2 * $MAX_REC))
	$oStmt.Execute()
Next
$oStmt.Close ()
$oConnection2.Execute("COMMIT TRANSACTION")


status_text("Executing log query #1. Progress event will be raised.")
$oConnection1.ProgressPeriod = $MAX_REC
$nAvg = $oConnection1.Execute("SELECT avg(b) FROM Test")
$oConnection1.ProgressPeriod = 0
status_text(StringFormat("Avg: %.2f.", $nAvg))
Sleep(200)

status_text("Executing log query #2. Progress event will be raised.")
$oConnection2.ProgressPeriod = $MAX_REC
$nTotal = $oConnection2.Execute("SELECT total(b) FROM Test")
$oConnection2.ProgressPeriod = 0
status_text(StringFormat("Total: %.2f.", $nTotal))
Sleep(200)

status_text("Shutdown.")
$g_oLiteBusy1 = 0
$g_oLiteProgress1 = 0
$g_oLiteBusy2 = 0
$g_oLiteProgress2 = 0
$oConnection1.Close ()
$oConnection2.Close ()

status_text("Cleanup.")
FileRecycle($DB_FILE)

status_text("Bye.")
Exit 0
