#comments-start

	This sample demonstrates how to use
	LiteX Automation together with "standard"
	_SQLite3_... UDFs included into AutoIt
	
	If you have LiteX Automation library registered you do not
	need nay additional versions of sqlite3.dll library in system or script
	directory. Location of sqlite3.dll library you can read from registry.
	This technique works even if you use Unicode version of LiteX Automation
	(e.g. sqlite3u.dll).
	
#comments-end

#include <File.au3>
#include "SQlite16.au3"

Global Const $DB_FILE = _TempFile()
Global Const $DB_QUERY = "SELECT count(*),avg(b),total(c) FROM Test"
Global Const $MAX_REC = 2222
Global Const $BIG_INT = "9223372036854775807"
Global $g_oComError

#comments-start
	Reads location of LiteX Automation library from registry.
#comments-end
Func _LiteX_Path()
	Local $sLiteXPath = RegRead("HKEY_CLASSES_ROOT\CLSID\{3E22694D-7B92-42A1-89A7-668E2F7AA107}\InprocServer32", "")
	If @error Then
		Return SetError(@error, @extended, "")
	Else
		Return SetError(0, 0, $sLiteXPath)
	EndIf
EndFunc   ;==>_LiteX_Path

#comments-start
	If LiteX Automation is registered in your system then you can use that library
	instead of standard sqlite3.dll.
	
	Instead of _SQLite_Startup use _LiteX_Startup.
	Call normal _SQLite_Shutdown() when you finish.
#comments-end
Func _LiteX_Startup()
	Local $sLiteXPath
	
	$sLiteXPath = _LiteX_Path()
	If Not @error Then
		_SQLite_Startup($sLiteXPath)
		If @error Then
			Return SetError(1, 0, False)
		Else
			Return SetError(0, 0, True)
		EndIf
	Else
		Return SetError(1, 1, False)
	EndIf
EndFunc   ;==>_LiteX_Startup

#comments-start
	Here is sample application.
#comments-end

Func com_err_func()
	Beep(400, 200)
	Dim $sMsg = "COM Error: " & Hex($g_oComError.number, 8)
	ConsoleWriteError($sMsg & @CRLF)
	$sMsg = "Source: " & $g_oComError.source
	ConsoleWriteError($sMsg & @CRLF)
	$sMsg = "Error description #1: " & $g_oComError.windescription
	ConsoleWriteError($sMsg & @CRLF)
	$sMsg = "Error  description #2: " & $g_oComError.description
	ConsoleWriteError($sMsg & @CRLF)
	If Not @Compiled Then
		$sMsg = "Line: " & $g_oComError.scriptline
		ConsoleWriteError($sMsg & @CRLF)
	EndIf
	Return SetError(1, $g_oComError, False)
EndFunc   ;==>com_err_func

$g_oComError = ObjEvent("AutoIt.Error", "com_err_func")

$sLiteXPath = _LiteX_Path()
If @error Then
	ConsoleWriteError("Fail to find LiteX Automation library." & @CRLF)
	ConsoleWriteError("Make sure that LiteX Automation is registered." & @CRLF)
	Exit 1
EndIf

ConsoleWrite("LiteX Automation library location: " & $sLiteXPath & @CRLF)
ConsoleWrite("LiteX Automation library version: " & FileGetVersion($sLiteXPath) & @CRLF)

;
; Init
;
$oConnection = ObjCreate("LiteX.LiteConnection")
If @error Then
	ConsoleWriteError("Fail to create LiteX Connection object." & @CRLF)
	Exit 10
EndIf

$oLi = ObjCreate("LiteX.LargeInteger")
If @error Then
	ConsoleWriteError("Fail to create LiteX LargeInteger object." & @CRLF)
	Exit 11
EndIf


_LiteX_Startup()
If @error Then
	ConsoleWriteError("Fail to load LiteX Automation library." & @CRLF)
	Exit 2
EndIf

;
; Version
;
ConsoleWrite("SQLite3 library version #1: " & _SQLite_LibVersion() & @CRLF)
ConsoleWrite("SQLite3 library version #2: " & $oConnection.Version (False) & @CRLF)

;
; Structure
;
ConsoleWrite("Creating database structure." & @CRLF)
$oConnection.Open ($DB_FILE)
$oConnection.BatchExecute ("CREATE TABLE Test( a INTEGER PRIMARY KEY, b, c );CREATE INDEX idxb ON Test(b);CREATE INDEX idxc ON Test(c)")
$oConnection.Execute("CREATE TABLE LTest( a INTEGER PRIMARY KEY)")

$db = _SQLite_Open16 ($DB_FILE)

;
; large integers
;
ConsoleWrite( "Testing large integers." & @CRLF )
$oLi.QuadPart = $BIG_INT
$oStmt = $oConnection.Prepare ("INSERT INTO LTest(a) VALUES (?)")
$oStmt.Parameters(0).Bind($oLi, 1)
$oStmt.Execute()
$oStmt.Close ()
ConsoleWrite(StringFormat("Large integer: %s = %.4f = (%x,%x)", $BIG_INT, $oLi.QuadPartCy, $oLi.highPart, $oLi.LowPart) & @CRLF)
ConsoleWrite(StringFormat("LastInsertRowid: %.0f", $oConnection.LastInsertRowid) & @CRLF)

$oStmt = $oConnection.Prepare( "SELECT last_insert_rowid()" )
$oStmt.Step()
$sLastId = $oStmt.Rows.Item.Item(0,4) ; ColumnValue(0,4) ; as text
$oStmt.Close()
ConsoleWrite( "LastInsertRowid again: " & $sLastId & @CRLF )


;
; Data phase one
;
ConsoleWrite("Building database phase 1 (UDF's)." & @CRLF)
$tBegin = TimerInit()
_SQLite_Exec($db, "BEGIN TRANSACTION")
For $i = 1 To $MAX_REC
	_SQLite_Exec($db, StringFormat("INSERT INTO Test(b,c) VALUES (%f,%d)", Random(88, 888, 1), Random(234, 444)))
Next
_SQLite_Exec($db, "COMMIT TRANSACTION")
$tDiff1 = TimerDiff($tBegin)
ConsoleWrite("Phase complete. Time " & Round($tDiff1, 3) & " ms." & @CRLF)

;
; Data phase two
;
ConsoleWrite("Building database phase 2 (LiteX Automation)." & @CRLF)
$tBegin = TimerInit()
$oConnection.Execute("BEGIN TRANSACTION")
$oStmt = $oConnection.Prepare ("INSERT INTO Test(b,c) VALUES (:one,:two)")
For $i = 1 To $MAX_REC
	$oStmt.BindParameters (Random(9, 999, 1), Random(432, 555))
	$oStmt.Execute()
Next
$oStmt.Close ()
$oConnection.Execute("COMMIT TRANSACTION")
$tDiff2 = TimerDiff($tBegin)
ConsoleWrite("Phase complete. Time " & Round($tDiff2, 3) & " ms." & @CRLF)
ConsoleWrite(StringFormat("Phase two was about %.1f times faster than phase one." & @CRLF, $tDiff1 / $tDiff2))

;
; Query using UDF's
;
ConsoleWrite("Executing query using UDF's: " & $DB_QUERY & @CRLF)
Dim $hQuery, $aRow
_SQLite_Query16 ($db, $DB_QUERY, $hQuery)
_SQLite_FetchData16 ($hQuery, $aRow)
_SQLite_QueryFinalize($hQuery)

ConsoleWrite("Query result #1: " & $aRow[0] & @CRLF)
ConsoleWrite("Query result #2: " & $aRow[1] & @CRLF)
ConsoleWrite("Query result #3: " & $aRow[2] & @CRLF)

;
; Query using LiteX Automation
;
ConsoleWrite("Executing query using LiteX Automation: " & $DB_QUERY & @CRLF)
$oStmt = $oConnection.Prepare ($DB_QUERY)
$oStmt.Step ()

ConsoleWrite("Query result #1: " & $oStmt.ColumnValue (0) & @CRLF)
ConsoleWrite("Query result #2: " & $oStmt.ColumnValue (1) & @CRLF)
ConsoleWrite("Query result #3: " & $oStmt.ColumnValue (2) & @CRLF)

$oStmt.Close ()
$oStmt = 0

;
; Collections
;
ConsoleWrite("Iterating records using collections. " & @CRLF)
$oStmt = $oConnection.Prepare ("SELECT ROWID,a,b,c FROM TEST ORDER BY c DESC LIMIT 3555,30")
For $oColumn In $oStmt.Columns
	ConsoleWrite("Column #" & $oColumn.Index & @TAB & $oColumn.Value & @CRLF)
Next

For $oRow In $oStmt.Rows (True, 20) ; try True or False as first parameter
	$sRow = "Row #"
	For $oVal In $oRow ; second collection
		$sRow &= $oVal
		$sRow &= @TAB
	Next
	ConsoleWrite(StringTrimRight($sRow, 1) & @CRLF)
Next
$oStmt.Close ()
$oStmt = 0

;
; Shutdown
;
ConsoleWrite("Shutdown." & @CRLF)
_SQLite_Close($db)
_SQLite_Shutdown()
$oConnection.Close ()

;
; Clean Up
;
FileRecycle($DB_FILE)

;
; Bye
;
ConsoleWrite("Bye." & @CRLF)
Exit 0