;
#include-once

#comments-start
	
	Extensions To UDF's included in SQLite.au3.
	Support For Unicode (UTF - 16LE).
	Currently AutoIt3 is Unicode application by default on NT-based systems.
	Using theese UDFs may be a little bit faster an allow you to avoid many
	text encoding problems.
	Remeber that by default database created by _SQLite_Open16 is UTF-16LE encoded.
	You may change new database encoding by PRAGMA CODEPAGE statement.
	
	_SQLite_Open16
	_SQLite_ErrMsg16
	_SQlite_Query16
	_SQLite_FetchData16
	_SQLite_FetchNames16
	
	__SQLite_ReportError16
	
#comments-end

#include <SQLite.au3>

;===============================================================================
;
; Function Name:      _SQLite_Open16()
; Description:        Opens a Database
; Parameter(s):       $sDatabase_Filename - Optional, Database Filename (uses ':memory:' db by default)
; Return Value(s):    Returns Database Handle
; @error Value(s):	   1 - Error Calling SQLite API 'sqlite3_open16'
;					  -1 - SQLite Reported an Error (Check @extended Value)
; @extended Value(s): Can be compared against $SQLITE_* Constants
; Author(s):          piccaso (Fida Florian), Edek
;
;===============================================================================
;
Func _SQLite_Open16($sDatabase_Filename = ":memory:")
	Local $avRval
	If IsKeyword($sDatabase_Filename) Then $sDatabase_Filename = ":memory:"
	$avRval = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_open16", "wstr", $sDatabase_Filename _ ; Database filename
			, "long*", 0) ; OUT: SQLite db handle
	If @error > 0 Then
		Return SetError(1, $SQLITE_MISUSE, 0)
	ElseIf $avRval[0] = $SQLITE_OK Then
		$g_hDB_SQLite = $avRval[2]
		__SQLite_hAdd ($avRval[2], $SQLITE_DBHANDLE)
		Return SetError(0, $avRval[0], $avRval[2])
	Else
		__SQLite_ReportError16($avRval[2], "_SQLite_Open16")
		_SQLite_Close($avRval[2])
		Return SetError(-1, $avRval[0], 0)
	EndIf
EndFunc   ;==>_SQLite_Open16


;===============================================================================
;
; Function Name:    _SQLite_ErrMsg16()
; Description:      Returns a String describing in English the error condition for the most recent sqlite3_* API call
; Parameter(s):     $hDB - Optional, An Open Database, Default is the Last Opened Database
; Return Value(s):  On Success - Returns Error message
; @error Value(s):	1 - Error Calling SQLite API 'sqlite3_errmsg16'
; Author(s):        piccaso (Fida Florian), Edek
;
;===============================================================================
;
Func _SQLite_ErrMsg16($hDB = -1)
	If Not __SQLite_hChk ($hDB, $SQLITE_DBHANDLE) = $SQLITE_OK Then Return "Library used incorrectly"
	Local $r = DllCall($g_hDll_SQLite, "wstr:cdecl", "sqlite3_errmsg16", "ptr", $hDB)
	If @error > 0 Then
		Return SetError(1, 0, "Library used incorrectly") ;DllCall Error
	Else
		Return $r[0]
	EndIf
EndFunc   ;==>_SQLite_ErrMsg16

;===============================================================================
;
; Function Name:    _SQlite_Query16()
; Description:      Prepares a SQLite Query
; Parameter(s):     $hDB - An Open Database, Use -1 To use Last Opened Database
;					$sSQL - SQL Statement to be executed
;					ByRef $hQuery - Passes out a Query Handle
; Return Value(s):  On Success - Returns $SQLITE_OK
;                   On Failure - Return Value can be compared against $SQLITE_* Constants
; @error Value(s):	-1 - SQLite Reported an Error (Check Return value)
;					 1 - Error Calling SQLite API 'sqlite3_prepare16_v2'
;					 2 - Call prevented by SafeMode
; Author(s):        piccaso (Fida Florian), Edek
;
;===============================================================================
;
Func _SQlite_Query16($hDB, $sSQL, ByRef $hQuery)
	If Not __SQLite_hChk ($hDB, $SQLITE_DBHANDLE) = $SQLITE_OK Then Return SetError(2, 0, $SQLITE_MISUSE)
	Local $iRval
	$iRval = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_prepare16_v2", _
			"ptr", $hDB, _
			"wstr", $sSQL, _
			"int", StringLen($sSQL) * 2, _
			"long*", 0, _
			"long*", 0)
	If @error > 0 Then
		Return SetError(1, 0, $SQLITE_MISUSE)
	ElseIf $iRval[0] = $SQLITE_OK Then
		$hQuery = $iRval[4]
		__SQLite_hAdd ($iRval[4], $SQLITE_QUERYHANDLE)
		Return $iRval[0]
	Else
		__SQLite_ReportError16($hDB, "_SQLite_Query16", $sSQL)
		Return SetError(-1, 0, $iRval[0])
	EndIf
EndFunc   ;==>_SQlite_Query16



;===============================================================================
;
; Function Name:    _SQLite_FetchData16()
; Description:      Fetches 1 Row of Data from a _SQLite_Query() based query
; Parameter(s):     $hQuery - Queryhandle passed out by _SQLite_Query()
;					ByRef $aRow - A 1 dimensional Array containing a Row of Data
;					$fBinary - Switch for Binary mode ($aRow will be a Array of Binary Strings)
; Return Value(s):  On Success - Returns $SQLITE_OK
;                   On Failure - Return Value can be compared against $SQLITE_* Constants
; @error Value(s):	-1 - SQLite Reported an Error (Check Return value)
;					 1 - Error Calling SQLite API 'sqlite3_step'
;					 2 - Error Calling SQLite API 'sqlite3_data_count'
;					 3 - Error Calling SQLite API 'sqlite3_column_text16'
;					 4 - Error Calling SQLite API 'sqlite3_column_type'
;					 5 - Error Calling SQLite API 'sqlite3_column_bytes'
;					 6 - Error Calling SQLite API 'sqlite3_column_blob'
;					 7 - Call prevented by SaveMode
; Author(s):        piccaso (Fida Florian), Edek
;
;===============================================================================
;
Func _SQLite_FetchData16($hQuery, ByRef $aRow, $fBinary = False)
	If Not __SQLite_hChk ($hQuery, $SQLITE_QUERYHANDLE) = $SQLITE_OK Then Return SetError(7, 0, $SQLITE_MISUSE)
	If Not IsArray($aRow) Then
		Dim $aRow[1]
	EndIf
	Local $iRval_Step, $iRval_ColCnt, $sRval, $i, $iRval_coltype
	Local $iColBytes, $vResult, $vResultStruct
	Local $SQLITE_NULL = 5
	If IsKeyword($fBinary) Then $fBinary = False
	$iRval_Step = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_step", "ptr", $hQuery)
	If @error > 0 Then
		Return SetError(1, 0, $SQLITE_MISUSE)
	ElseIf $iRval_Step[0] = $SQLITE_ROW Then
		$iRval_ColCnt = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_data_count", "ptr", $hQuery)
		If @error > 0 Then
			Return SetError(2, 0, $SQLITE_MISUSE)
		EndIf
		If $iRval_ColCnt[0] > 0 Then
			ReDim $aRow[$iRval_ColCnt[0]]
			For $i = 0 To $iRval_ColCnt[0] - 1
				If Not $fBinary Then
					$iRval_coltype = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_column_type", "ptr", $hQuery, "int", $i)
					If @error > 0 Then
						Return SetError(4, 0, $SQLITE_MISUSE)
					EndIf
					If $iRval_coltype[0] = $SQLITE_NULL Then
						$aRow[$i] = ""
						ContinueLoop
					EndIf
					$sRval = DllCall($g_hDll_SQLite, "wstr:cdecl", "sqlite3_column_text16", "ptr", $hQuery, "int", $i)
					If @error > 0 Then
						Return SetError(3, 0, $SQLITE_MISUSE)
					EndIf
					$aRow[$i] = $sRval[0]
				Else
					$iColBytes = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_column_bytes16", "ptr", $hQuery, "int", $i)
					If @error > 0 Then
						Return SetError(5, 0, $SQLITE_MISUSE)
					EndIf
					$vResult = DllCall($g_hDll_SQLite, "ptr:cdecl", "sqlite3_column_blob", "ptr", $hQuery, "int", $i)
					If @error > 0 Then
						Return SetError(6, 0, $SQLITE_MISUSE)
					EndIf
					$vResultStruct = DllStructCreate("byte[" & $iColBytes[0] & "]", $vResult[0])
					$aRow[$i] = Binary(DllStructGetData($vResultStruct, 1))
				EndIf
			Next
			Return $SQLITE_OK
		Else
			Return SetError(-1, 0, $SQLITE_EMPTY)
		EndIf
	Else ; incl. $SQLITE_DONE
		_SQLite_QueryFinalize($hQuery)
		If $iRval_Step[0] <> $SQLITE_OK Then SetError(-1)
		Return $iRval_Step[0]
	EndIf
EndFunc   ;==>_SQLite_FetchData16

;===============================================================================
;
; Function Name:    _SQLite_FetchNames16()
; Description:      Read out the Tablenames of a _SQLite_Query() based query
; Parameter(s):     $hQuery	- Query Handle Generated by SQLite_Query()
;                   ByRef $aNames - 1 Dimensional Array Containign the Table Names
; Return Value(s):  On Success - Returns $SQLITE_OK
;                   On Failure - Return Value can be compared against $SQLITE_* Constants
; @error Value(s):	-1 - SQLite Reported an Error (Check Return value)
;					 1 - Error Calling SQLite API 'sqlite3_data_count'
;					 2 - Error Calling SQLite API 'sqlite3_column_name16'
;					 3 - Call prevented by SaveMode
; Author(s):        piccaso (Fida Florian), Edek
;
;===============================================================================
;
Func _SQLite_FetchNames16($hQuery, ByRef $aNames)
	If Not __SQLite_hChk ($hQuery, $SQLITE_QUERYHANDLE) = $SQLITE_OK Then Return SetError(3, 0, $SQLITE_MISUSE)
	Local $avDataCnt, $avColName, $iCnt
	If Not IsArray($aNames) Then Dim $aNames[1]
	$avDataCnt = DllCall($g_hDll_SQLite, "int:cdecl", "sqlite3_column_count", "ptr", $hQuery)
	If @error > 0 Then
		Return SetError(1, 0, $SQLITE_MISUSE) ; DllCall Error (sqlite3_data_count)
	ElseIf $avDataCnt[0] > 0 Then
		ReDim $aNames[$avDataCnt[0]]
		For $iCnt = 0 To $avDataCnt[0] - 1
			$avColName = DllCall($g_hDll_SQLite, "wstr:cdecl", "sqlite3_column_name16", "ptr", $hQuery, "int", $iCnt)
			If @error > 0 Then
				Return SetError(2, 0, $SQLITE_MISUSE); DllCall Error (sqlite3_column_name)
			EndIf
			$aNames[$iCnt] = $avColName[0]
		Next
		Return $SQLITE_OK
	Else
		Return SetError(-1, 0, $SQLITE_EMPTY)
	EndIf
EndFunc   ;==>_SQLite_FetchNames16

#region	SQLite16.au3 Internal Functions

Func __SQLite_ReportError16($hDB, $sFunction, $sQuery = Default, $sError = Default, $vReturnValue = Default)
	If @Compiled Then Return
	If IsKeyword($sError) Then $sError = _SQLite_ErrMsg16($hDB)
	If IsKeyword($sQuery) Then $sQuery = ""
	Local $sOut
	$sOut &= "!   SQLite.au3 Error" & @LF
	$sOut &= "--> Function: " & $sFunction & @LF
	If $sQuery <> "" Then $sOut &= "--> Query:    " & $sQuery & @LF
	$sOut &= "--> Error:    " & $sError & @LF
	ConsoleWrite($sOut & @LF)
	If Not IsKeyword($vReturnValue) Then Return $vReturnValue
EndFunc   ;==>__SQLite_ReportError16

#endregion 	SQLite16.au3 Internal Functions